package Hush;
use strict;
use warnings;

my $VERSION = '0.02';

=head1 NAME

Hush - Hush applications

=head1 DESCRIPTION

This is the top-level namespace for Hush applications, which mostly consist of Hush::List currently.

=head1 SYNOPSIS

    use Hush;
    use Hush::List;

=head1 LATEST CODE

The latest released version can always be found at https://metacpan.org/module/Hush and
the git repository lives at https://github.com/leto/hushlist .

=head1 AUTHOR

Duke Leto <duke@leto.net>

=head1 SUPPORT THIS WORK

If you would like to support this work, because it helped you out
and your life is now amazing, feel free to send some donations to
the developer via the blockchain :

HUSH: zcB3LvCwYPdvVmNmTvEFdFGwb4teocrPnVD8wvBgMn8ZKo5Vq9ypJLpAuqUM31hEbBnSHCjCpax5mjUi2HEM6NB8x7vfKrV

ZEC: zcZLVdeNHvbw58ch56RWi92ws8hweLHyxhoT6jniFKd8kkBPXPR5E46YXzAqXhrfagtwRojAtumg4M3kmrHfZPU6m63Rj5z


Cryptocoins make the world go round and round.

=head1 SEE ALSO

Hush::List, Alien::Hush

=head1 COPYRIGHT

Copyright (c) 2017-2018 by Duke Leto <duke@leto.net>.  All rights reserved.

=head1 LICENSE AGREEMENT

This package is free software; you can redistribute it and/or
modify it under the GNU Public License Version 3.

=cut

1;
